%% Run this file with the desired maximal mathematical order to find the 
% optimal order of the model

max_order = 30;

ident_set = 'ident_wp.mat';
test_set = 'test_wp.mat';

load(test_set);
p_c = choke_pressure;
v_d = piston_velocity_ref;

id_data = extract_data(ident_set);
test_data = extract_data(test_set);

% Updated version, p_dh NOT included in black-box modeling (Attempt on grey-box modeling)
% U = [choke opening [0-1], piston velocity reference]
% Y = [pipe pressures (10-1)] 
U = [id_data(:,1) id_data(:,2)];
Y = id_data(:,3:(end-2));
U_test = [test_data(:,1) test_data(:,2)];
Y_test = test_data(:,3:(end-2));

figure(300); 
title('Data sets used in system identification'); 
subplot(2,2,1); 
plot(U); 
title('U, id'); 
xlabel('Time, [s]');
legend('p_c', 'v_d');
subplot(2,2,3); 
plot(U_test); 
title('U, test'); 
xlabel('Time, [s]');
legend('p_c', 'v_d');
subplot(2,2,2); 
plot(Y); 
title('Y, id'); 
subplot(2,2,4); 
plot(Y_test); 
title('Y, test');

[~, length_y] = size(Y);

dsr_error = zeros(max_order, max_order, length_y);
sum_dsr_error = zeros(max_order, max_order);
is_stable = zeros(max_order, max_order);

sample = 0.01;
end_time = (length(U_test)-1)/100;
t = 0:0.01:end_time;

for temp_max_order = 1:max_order
    for order = 1:temp_max_order
        [A,B,C,D,CF,F,~]=dsr(Y,U,temp_max_order,1,temp_max_order,1,order);
        K = (A\CF)/F;
        R = F*F'; 
        sysd = ss(A, B, C, D, 0.01);
        Y_p = lsim(sysd, U_test, t);
        figure(1); plot(Y_p)
        lambda = eig(A);
        if(max(lambda)>1 || min(lambda)<-1)
            text = ['System ', num2str(order), ', ', num2str(temp_max_order), ' is unstable.\n'];
            fprintf(text);
            is_stable(temp_max_order, order) = 0;
        else
            is_stable(temp_max_order, order) = 1;
        end
        RMSE_Y_p = sqrt(mean((Y_test-Y_p).^2));
        dsr_error(temp_max_order, order, :) = RMSE_Y_p;    
        sum_dsr_error(temp_max_order, order) = sum(RMSE_Y_p);
        clear A B C D CF F K R Y_p
    end
end

sum = 10000000000;
for plot_nr = 1:max_order
    [sum, index] = min(nonzeros(sum_dsr_error(plot_nr,:)));
    if plot_nr == 1
        min_sum = sum;
        optimal_order = index;
        optimal_max_order = plot_nr;
    end
    if sum < min_sum
        min_sum = sum;
        optimal_order = index;
        optimal_max_order = plot_nr;
    end
end
    
for plot_nr = 1:max_order
    figure(plot_nr)
    scatter(1:30, sum_dsr_error(plot_nr,:), '.');
    plot_title = ['Max order = ', num2str(plot_nr)];
    title(plot_title)
    xlabel('Order')
    ylabel('Pressure in bar')
    axis([0,30,0,30])
    legend('Sum dsr-error')
end

save('dsr_error.mat', 'dsr_error', 'sum_dsr_error', 'min_sum', ...
    'optimal_order', 'optimal_max_order', 'is_stable')

clearvars -except optimal_max_order optimal_order