function online_update_mpc(block)
%   Copyright 2003-2010 The MathWorks, Inc.
setup(block);

%endfunction

function setup(block)

load('dimensions.mat');
% Dim. of estimated disturbance series: mpc_horizon (or dim_v_d)
% Dim. of system: order


% Register number of ports
block.NumInputPorts  = 3;
block.NumOutputPorts = 1;

% Setup port properties to be inherited or dynamic
block.SetPreCompInpPortInfoToDynamic;
block.SetPreCompOutPortInfoToDynamic;

%%Referansesignal - P_dh,ref
  block.InputPort(1).Dimensions        = mpc_horizon;         
  block.InputPort(1).DirectFeedthrough = false;
%%Forstyrrelse - v_d_est
  block.InputPort(2).Dimensions        = mpc_horizon;
  block.InputPort(2).DirectFeedthrough = false;
%%x_hat fra Kalmanfilter
  block.InputPort(3).Dimensions        = order;
  block.InputPort(3).DirectFeedthrough = false;
  
block.OutputPort(1).Dimensions       = 1;

% Register parameters
block.NumDialogPrms     = 0;

% Register sample times
block.SampleTimes = [qp_sample 0];
% block.SampleTimes = [0.2 0];

% Specify the block simStateCompliance. The allowed values are:
%    'DefaultSimState', < Same sim state as a built-in block
block.SimStateCompliance = 'DefaultSimState';


block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
block.RegBlockMethod('InitializeConditions', @InitializeConditions);
block.RegBlockMethod('Outputs', @Outputs);
block.RegBlockMethod('Update', @Update);

%end setup

function DoPostPropSetup(block)
block.NumDworks = 2;
  
  block.Dwork(1).Name            = 'u_delta';
  block.Dwork(1).Dimensions      = 1;
  block.Dwork(1).DatatypeID      = 0;      % double
  block.Dwork(1).Complexity      = 'Real'; % real
  block.Dwork(1).UsedAsDiscState = true;
    
  block.Dwork(2).Name = 'u';
  block.Dwork(2).Dimensions = 1;
  block.Dwork(2).DatatypeID = 0;
  block.Dwork(2).Complexity = 'Real';
%endfunction

function InitializeConditions(block)
load('p_c_init.mat')
block.Dwork(1).Data = 0;
block.Dwork(2).Data = p_c_L;
%end InitializeConditions


function Outputs(block)
load('p_c_init.mat')
block.OutputPort(1).Data = p_c_L + block.Dwork(2).Data;

%end Outputs
function Update(block)
load('setup_mpc.mat');
% Initial condition, x_hat from kalman filter.
x0 = [block.InputPort(3).Data; block.Dwork(2).Data];

% Disturbance from input: 
d = 2*block.InputPort(2).Data;

% Calculation of reference output, y_ref (= p1_ref): 
% If running the test_mpc.slx: remember to set the reference in pascal OR
% remove the 1e-5 in the following line 
y1_ref = block.InputPort(1).Data*1e-5 - x_L(1);

% For details, see report ch.3 (iallfall til s lenge, endre denne)
Y_dev = A_hat*x0 + E_hat*d - y1_ref;
c_transposed = Y_dev'*Q_hat*B_hat;

% Lower and upper bounds for u
lb = -1*ones(mpc_horizon,1); 
ub = 1*ones(mpc_horizon,1);

Y_l = -inf;%x_L(1);
Y_u = 10-x_L(1);

L = [B_hat; -B_hat];
b = [Y_u-A_hat*x0-E_hat*d;-Y_l+A_hat*x0+E_hat*d];
% % Constraint definition:
% b = [-Ul_hat; Uu_hat; -Yl_hat+y1_ref; Yu_hat-y1_ref] + ...
%     [zeros(mpc_horizon,1); zeros(mpc_horizon,1); Y_dev; -Y_dev];
% b1 = [-Ul_hat; Uu_hat];
% b = [-Ul_hat+u_ref; Uu_hat-u_ref; -Yl_hat+y1_ref; ...
%     Yu_hat-y1_ref]; 

% % Solving with "full" constrains
% [v, f_val, exitflag] = quadprog(H_tilde,c_transposed',L1,b1,[],[],lb,ub,[],opt);

% Solving with only upper- and lower bounds on u
[v, f_val, exitflag] = quadprog(H_tilde,c_transposed',L,b,[],[],lb,ub,[],opt);
exitflag

% Output: 
block.Dwork(1).Data = v(1);
block.Dwork(2).Data = block.Dwork(2).Data + v(1);


%end Update


%%
%% Terminate:
%%   Functionality    : Called at the end of simulation for cleanup
%%   Required         : Yes
%%   C-MEX counterpart: mdlTerminate
%%
function Terminate(block)

%end Terminate

