%% Setup for testing of the MPC controller

order = 7;
model = ['30-Apr-2014_model_order_', num2str(order)];
load(model);

%% Model setup for kalman filter
sysd = ss(A, B, C, D, 0.01);
sysc = d2c(sysd);

[A2, B2, C2, D2] = ssdata(sysc);
%% MPC setup
% MPC horizon
mpc_horizon = 100;
period = 10;


% Sample time for QP problem
qp_sample = 0.1;

sysd = c2d(sysc, qp_sample);
% sysd = ss(A, B, C, D, 0.02);
[A, B, C, D] = ssdata(sysd);

%% Weighting matrices: 
y_final_weight = 4000;
y_weight = 2000;
Q_hat = blkdiag2(y_weight, mpc_horizon-1);
Q_hat = blkdiag(Q_hat, y_final_weight);
u_weight = 2000;
P_hat = blkdiag2(u_weight, mpc_horizon);

%%With integral action: 
% Model setup
[nx, ~] = size(B);
[ny, ~] = size(C);
nu = 1;
% Model that is using input changes instead of the input itself:  
% x_tilde_k+1 = A_tilde*x_tilde_k + B_tilde*delta_u_k + E_tilde*d_k
% y_k = C_tilde*x_tilde_k
A_tilde = [A B(:,1); zeros(nu, nx), eye(nu)];
B_tilde = [B(:,1); eye(nu)];
C_tilde = [C zeros(ny,nu)];%; zeros(nu,nx) eye(nu)];
E_tilde = [B(:,2); zeros(nu,nu)];
C1 = C_tilde(1,:);

%% MPC setup
% A_hat = [A; A^2; --- ; A^mpc_horizon]
A_hat = zeros(mpc_horizon, (nx+nu));
for i = 1:mpc_horizon
    A_hat(i,:) = C1*A_tilde^i;
end
% B_hat = [B 0 --- 0; AB B --- 0; A^2B AB B ---; --- ; A^mpc_horizon-1*B --- B];
B_hat = zeros(mpc_horizon, mpc_horizon);
for i = 1:mpc_horizon
    for j = 1:i
        B_hat(i,j) ...
            = C1*A_tilde^(i-j)*B_tilde;
    end
end

% B_d_hat = E_hat = same as B_hat, switch B with E: 
E_hat = zeros(mpc_horizon, mpc_horizon);
for i = 1:mpc_horizon
    for j = 1:i
        E_hat(i,j) ...
            = C1*A_tilde^(i-j)*E_tilde;
    end
end

H_tilde = (B_hat'*Q_hat*B_hat + P_hat);
H_tilde = (H_tilde+H_tilde')/2;

%% Constraints: 
% Ul = 0;
% Uu = 25; 


%% Deviation variable constructors (working points)
% Working point definition: 
% choke_wp = 0.59;
p_dh_L = 4.58;
p_c_L = 4.64;
x_L = [4.64;4.62;4.59;4.57;4.54;4.52;4.49;4.47;4.44];
u_L = [p_c_L; 0];

u_ref = 0;
u_ref = blkvector(u_ref,mpc_horizon); 

% Setup for the quadprog-function in the s-function 
opt = optimoptions('quadprog','Algorithm','active-set','Display','off');



save('setup_mpc.mat','A_hat','B_hat', 'E_hat', 'Q_hat',...
    'H_tilde','mpc_horizon','C1','qp_sample', 'x_L', ...%'Ul_hat', 'Uu_hat', ...
    'opt', 'p_c_L', 'P_hat');
save('p_c_init.mat', 'p_c_L');

save('dimensions.mat', 'order', 'qp_sample', 'mpc_horizon');

%% Disturbance observer setup
% Define the period of the disturbance: (worst case: 3) 
% Uncomment if script is not used! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% period = 10;
omega = (2*pi)/period;
A_d = [0 omega;
  -omega 0];
C_d = [1 0];

p_piston = [-0.7; -0.5];   

L_piston = place(A_d,C_d',p_piston).';

% Test of the eigenvalues: If eigenvalues over 0, unstable disturbance
% model. Choose other poles. 
display('Eigenvalues of disturbance estimate:');
v_eig = eig(A_d-L_piston*C_d);
display(v_eig)
if sum(v_eig >= 0)
    display('Error initModel.m: Postive eigenvalues in disturbance estimate.')
    display('Please change the desired pole placement. (p_piston)');
end
t = 0:qp_sample:(mpc_horizon*qp_sample)-qp_sample;
B_d = [0;0];
sys_dist = ss(A_d, B_d, C_d, 0);
save('disturbance_data.mat', 'sys_dist', 'mpc_horizon', 't')

% Setup for disturbance observer: 
A_d = A_d - L_piston*C_d;
B_d = L_piston;
