%% Run this file in the library folder with the file "Examples/forFreqResp.slx" open to get the frequency response of the simulator.

load('Examples/realresp.mat')

TSim = 900; 
Fs = 100;
T = 1/Fs;

L = TSim*1/T;                           % Length of signal
t = 0:T:TSim;                           % Time vector

NFFT = 2^nextpow2(L);                   % Next power of 2 from length of y
f = Fs/2*linspace(0,1,NFFT/2+1);

choke = [0 1];

amp = zeros(length(choke), length(f));
ang = zeros(length(choke), length(f));

for i=1:length(choke)
    opening = choke(i);

    sim('forFreqResp', TSim);

    u = pistonPosSim.Data;
    y = P1Sim.Data/100000;   

    Y = fft(y-mean(y),NFFT)/L;
    U = fft(u-mean(u),NFFT)/L;

    A = abs(Y(1:NFFT/2+1)./U(1:NFFT/2+1));
    phi = unwrap(angle(Y(1:NFFT/2+1))-angle(U(1:NFFT/2+1)));

    amp(i,:) = A;
    ang(i,:) = phi;
end

%% CLOSED CHOKE PLOT
i = 1;
set(0,'DefaultAxesLineStyleOrder',{'-',':','--'})
set(0,'DefaultAxesColorOrder',[0 0 0; 1 0 0; 0 0 1]);

subplot(221)
plot(f,smooth(amp(i,:),20), 'Linewidth', 2);
hold all;
plot(realF, closedRealA, '--r', 'linewidth', 2)

xlabel('Frequency (Hz)')
ylabel('Magnitude [bar/m]')
xlim([0.1 0.6])
ylim([0 50])

subplot(223)
plot(f(100:end),180/pi*unwrap(smooth(ang(i,100:end)-round(ang(i,100:end)/(2*pi))*2*pi,20)), 'Linewidth', 2); 
hold all;
plot(realF, closedRealP, '--r', 'linewidth', 2)

xlabel('Frequency (Hz)')
ylabel('Phase shift [deg]')
xlim([0.1 0.6])
ylim([-200 0])

%% OPEN CHOKE PLOT
i = 2;
subplot(222)
plot(f,smooth(amp(i,:),20), 'Linewidth', 2) 
hold all;

xlabel('Frequency (Hz)')
ylabel('Magnitude [bar/m]')
xlim([0.1 0.6])
ylim([0 50])

plot(realF, openRealA, '--r', 'linewidth', 2)

subplot(224)
plot(f(100:end),180/pi*unwrap(smooth(ang(i,100:end)-round(ang(i,100:end)/(2*pi))*2*pi,20)), 'Linewidth', 2); 

hold all;
plot(realF, openRealP, '--r', 'linewidth', 2)

xlabel('Frequency (Hz)')
ylabel('Phase shift [deg]')
xlim([0.1 0.6])
ylim([-200 0])

hL =legend('Simulator', 'Actual');

% Programatically move the legend
newPosition = [0.4 0.4 0.2 0.2];
newUnits = 'normalized';
set(hL,'Position', newPosition,'Units', newUnits);
set(hL,'FontSize',8);
set(hL,'units','pixels');
set(hL,'outerposition',[230,180,100,70]);