function [p1, p2, G] = kernelsolver(p, omega, l, F1, rho, c, A2)
    % Kernel solver
    k = l*F1/(rho*c);

    %%generate grid
    N = 100;
    dx = 1/(N-1);
    dxi= 1/(N-1);

    Pvu = zeros(N,N);
    Puu = zeros(N,N);

    % Antidiagonal:
    % calculate Pvu on the antidiagonal
    for i=1:N
        Pvu(N-i+1,i) = 0.25*k*exp(-k*(dx*(i-1)));
    end

    % calculating Puu on the antidiagonal
    Puu(N,1) = -1*Pvu(N,1);
    for i=2:N
        Puu(N-i+1,i) = Puu(N-i+2,i-1) + dx*(-(k/2)*exp(k*(dx*(i-1)))*Pvu(N-i+2,i-1));
    end

    % iterating over the grid as described in the report
    for i=2:N
        for(j=1:(N-i+1))
            pvuprev = (Pvu(N-j+1, j+i-2)+Pvu(N-j, j+i-1))/2;
            puuprev = (Puu(N-j+1, j+i-2)+Puu(N-j, j+i-1))/2;
            Pvu(N-j+1, j+i-1) = pvuprev - (dx/2)*(k/2*exp(-k*(dx*(i+j)))*puuprev);
        end
        Puu(N,i) = -Pvu(N,i);
        for(j=2:(N-i+1))
            Puu(N-j+1, j+i-1) = Puu(N-j+2, j+i-2) + dx*(-k/2*exp(k*(dx*(i+j-1)))*Pvu(N-j+1, j+i-1));
        end
    end

    %% Calculating observer gains
    A = [0 -omega; omega 0];
    C = -A2*[0 1];

    L = place(A',C',p).';
    eig(A-L*C);

    d_alpha = l/c;
    G = expm(A*d_alpha)*L;

    % P1
    x=0:dx:1;
    xi=0:dxi:1;
    for i=1:length(x)
        integral1 = 0;
        integral2 = 0;
        for j=i:length(xi)
            integral1 = integral1 + dxi*Puu(i, j)*C*expm(A*(1-xi(j))*l/c)*L;
            integral2 = integral2 + dxi*Pvu(i, j)*C*expm(A*(1-xi(j))*l/c)*L;
        end
        p1(i) = C*expm(A*(1-x(i))*l/c)*L -(c/l)*Puu(i, N)-integral1;
        p2(i) = -(c/l)*Pvu(i,N) -integral2;
    end
end
